//
//  AutoLaunchController.m
//  AsReaderScan
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "AutoLaunchController.h"

@implementation AutoLaunchController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupUI];
    self.title = @"Bundle ID";
    self.appBundleIdTextField.text = [AsReaderInfo sharedInstance].bundleIdentifier;

    [[NSNotificationCenter defaultCenter] 
		addObserver:self
        selector:@selector(textFieldDidChange:)
        name:UITextFieldTextDidChangeNotification
        object:self.readerBundleIdTextField];
		
    [self updateButtonState];
}

- (void)setupUI {
    self.view.backgroundColor = [UIColor whiteColor];
    
    self.bundleIdTitleLabel = [[UILabel alloc] init];
    self.bundleIdTitleLabel.translatesAutoresizingMaskIntoConstraints = NO;
    self.bundleIdTitleLabel.font = [UIFont boldSystemFontOfSize:25];
    self.bundleIdTitleLabel.textColor = [UIColor blackColor];
    self.bundleIdTitleLabel.text = @"Bundle ID";
    
    self.appBundleIdLabel = [[UILabel alloc] init];
    self.appBundleIdLabel.translatesAutoresizingMaskIntoConstraints = NO;
    self.appBundleIdLabel.font = [UIFont systemFontOfSize:25];
    self.appBundleIdLabel.textColor = [UIColor blackColor];
    self.appBundleIdLabel.text = @"App’s Bundle ID:";
    
    self.appBundleIdTextField = [[UILabel alloc] init];
    self.appBundleIdTextField.translatesAutoresizingMaskIntoConstraints = NO;
    self.appBundleIdTextField.font = [UIFont systemFontOfSize:25];
    self.appBundleIdTextField.textColor = [UIColor blackColor];
    
    self.readerBundleIdLabel = [[UILabel alloc] init];
    self.readerBundleIdLabel.translatesAutoresizingMaskIntoConstraints = NO;
    self.readerBundleIdLabel.font = [UIFont systemFontOfSize:25];
    self.readerBundleIdLabel.textColor = [UIColor blackColor];
    self.readerBundleIdLabel.text = @"Bundle ID stored in the reader:";
    
    self.readerCurrentBundleIdLabel = [[UILabel alloc] init];
    self.readerCurrentBundleIdLabel.translatesAutoresizingMaskIntoConstraints = NO;
    self.readerCurrentBundleIdLabel.font = [UIFont systemFontOfSize:25];
    self.readerCurrentBundleIdLabel.textColor = [UIColor blackColor];
    
    self.readerBundleIdTextField = [[UITextField alloc] init];
    self.readerBundleIdTextField.translatesAutoresizingMaskIntoConstraints = NO;
    self.readerBundleIdTextField.font = [UIFont systemFontOfSize:25];
    self.readerBundleIdTextField.textColor = [UIColor blackColor];
    self.readerBundleIdTextField.borderStyle = UITextBorderStyleRoundedRect;

    self.okButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.okButton setTitle:@"Save" forState:UIControlStateNormal];
    [self.okButton addTarget:self action:@selector(okButtonTapped) forControlEvents:UIControlEventTouchUpInside];
    self.okButton.translatesAutoresizingMaskIntoConstraints = NO;

    self.cancelButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.cancelButton setTitle:@"Cancel" forState:UIControlStateNormal];
    [self.cancelButton addTarget:self action:@selector(cancelButtonTapped) forControlEvents:UIControlEventTouchUpInside];
    self.cancelButton.translatesAutoresizingMaskIntoConstraints = NO;

    self.appCopyButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.appCopyButton setTitle:@"Copy" forState:UIControlStateNormal];
    [self.appCopyButton addTarget:self action:@selector(appCopyButtonTapped) forControlEvents:UIControlEventTouchUpInside];
    self.appCopyButton.translatesAutoresizingMaskIntoConstraints = NO;
    self.appCopyButton.backgroundColor = [UIColor blackColor];

    self.readerCopyButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.readerCopyButton setTitle:@"Copy" forState:UIControlStateNormal];
    [self.readerCopyButton addTarget:self action:@selector(readerCopyButtonTapped) forControlEvents:UIControlEventTouchUpInside];
    self.readerCopyButton.translatesAutoresizingMaskIntoConstraints = NO;
    self.readerCopyButton.backgroundColor = [UIColor blackColor];

    self.swAction = [[UISwitch alloc] init];
    self.swAction.translatesAutoresizingMaskIntoConstraints = NO;
    self.swAction.on = NO;

    [self.view addSubview:self.bundleIdTitleLabel];
    [self.view addSubview:self.appBundleIdLabel];
    [self.view addSubview:self.appBundleIdTextField];
    [self.view addSubview:self.readerBundleIdLabel];
    [self.view addSubview:self.readerCurrentBundleIdLabel];
    [self.view addSubview:self.readerBundleIdTextField];
    [self.view addSubview:self.appCopyButton];
    [self.view addSubview:self.readerCopyButton];
    [self.view addSubview:self.okButton];
    [self.view addSubview:self.cancelButton];
    [self.view addSubview:self.swAction];

    // Set up constraints
    [NSLayoutConstraint activateConstraints:@[
        [self.bundleIdTitleLabel.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:70],
        [self.bundleIdTitleLabel.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.bundleIdTitleLabel.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        
        [self.appBundleIdLabel.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:120],
        [self.appBundleIdLabel.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.appBundleIdLabel.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        
        [self.appBundleIdTextField.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:150],
        [self.appBundleIdTextField.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.appBundleIdTextField.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.appBundleIdTextField.heightAnchor constraintEqualToConstant:60],
        
        [self.readerBundleIdLabel.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:250],
        [self.readerBundleIdLabel.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.readerBundleIdLabel.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        
        [self.readerCurrentBundleIdLabel.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:290],
        [self.readerCurrentBundleIdLabel.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.readerCurrentBundleIdLabel.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        
        [self.readerBundleIdTextField.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:350],
        [self.readerBundleIdTextField.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.readerBundleIdTextField.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.readerBundleIdTextField.heightAnchor constraintEqualToConstant:60],
        
        [self.appCopyButton.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:150],
        [self.appCopyButton.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.appCopyButton.heightAnchor constraintEqualToConstant:40],
        [self.appCopyButton.widthAnchor constraintEqualToConstant:80],
        
        [self.readerCopyButton.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:290],
        [self.readerCopyButton.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.readerCopyButton.heightAnchor constraintEqualToConstant:40],
        [self.readerCopyButton.widthAnchor constraintEqualToConstant:80],
        
        [self.swAction.topAnchor constraintEqualToAnchor:self.readerBundleIdTextField.bottomAnchor constant:10],
        [self.swAction.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],

        [self.okButton.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:600],
        [self.okButton.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.okButton.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.okButton.heightAnchor constraintEqualToConstant:40],

        [self.cancelButton.topAnchor constraintEqualToAnchor:self.okButton.bottomAnchor constant:10],
        [self.cancelButton.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.cancelButton.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.cancelButton.heightAnchor constraintEqualToConstant:40]
    ]];
    
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTap)];
    tapGesture.cancelsTouchesInView = YES;
    [self.view addGestureRecognizer:tapGesture];
}

- (void)textFieldDidChange:(NSNotification *)notification {
    [self updateButtonState];
}

- (void)updateButtonState {
    if (self.readerBundleIdTextField.text.length > 0) {
        self.okButton.enabled = YES;
    } else {
        self.okButton.enabled = NO;
    }
}

- (void)okButtonTapped {
    if (self.readerBundleIdTextField.text.length == 0) {
        return;
    }
    [self.delegate setAutoLaunch:self.swAction.isOn bundle:self.readerBundleIdTextField.text];
    [self dismissViewControllerAnimated:YES completion:nil];
    self.readerBundleIdTextField.text = @"";
    [self updateButtonState];
}

- (void)cancelButtonTapped {
    [self dismissViewControllerAnimated:YES completion:^{
        [self updateButtonState];
    }];
    self.readerBundleIdTextField.text = @"";
}

- (void)appCopyButtonTapped {
    self.readerBundleIdTextField.text = self.appBundleIdTextField.text;
    [self updateButtonState];
}

- (void)readerCopyButtonTapped {
    self.readerBundleIdTextField.text = self.readerCurrentBundleIdLabel.text;
    [self updateButtonState];
}

- (void)handleTap {
    [self.view endEditing:YES];
}

@end

// ALPresenterDelegate methods
@implementation AutoLaunchController (ALPresenterDelegate)

- (void)setAutoLaunch:(BOOL)isOn bundle:(NSString *)bundle {
    // Implement the method
}

- (void)receivedGetAutoLaunch:(BOOL)isOn bundle:(NSString *)bundle {
    dispatch_async(dispatch_get_main_queue(), ^{
        self.readerCurrentBundleIdLabel.text = bundle;
        [self.swAction setOn:isOn animated:YES];
    });
}

// - (void)receivedSetAutoLaunch:(BOOL)isOn bundle:(NSString *)bundle {
//     dispatch_async(dispatch_get_main_queue(), ^{
//         self.readerBundleIdTextField.text = bundle;
//         [self.swAction setOn:isOn animated:YES];
//     });
// }

@end
