//
//  ReaderModel.h
//  AsReaderScan
//
//  @brief  This is for M24D. This file defines the ReaderInfo class, which encapsulates various settings and preferences related to the reader's functionality in the app.
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

#ifndef ReaderModel_h
#define ReaderModel_h


#endif /* ReaderModel_h */
// ReaderModel.h
#import <Foundation/Foundation.h>

typedef NS_ENUM(NSInteger, SleepTime) {
    Min5,
    Min30,
    Min60,
    Min120,
    NonSleep
};

typedef NS_ENUM(NSInteger, CodeID) {
    CodeIDNone,
    CodeIDAim,
    CodeIDSymbol
};

typedef NS_ENUM(NSInteger, TypeOCR) {
    OCRNone,
    OCRA,
    OCRB,
    OCRAB
};

typedef NS_ENUM(NSInteger, SyncSetting) {
    SyncStart,
    SyncBeepSetting,
    SyncSleepBeep,
    SyncCodeID
};

@interface ReaderInfo : NSObject

@property (nonatomic, assign) NSInteger battery;
@property (nonatomic, assign) CodeID codeID;
@property (nonatomic, assign) NSInteger selectItem;
@property (nonatomic, assign) NSInteger sleepTime;

@property (nonatomic, assign) BOOL appBeep;
@property (nonatomic, assign) BOOL triggerMode;
@property (nonatomic, assign) BOOL ssiBeep;
@property (nonatomic, assign) BOOL continuousReading;

- (instancetype)init;

// SleepTime methods
+ (NSInteger)sleepTimeFromIndex:(NSInteger)index;
+ (NSString *)sleepTimeStringFromIndex:(NSInteger)index;
+ (NSArray<NSString *> *)sleepTimeArray;

// CodeID methods
+ (CodeID)codeIDFromIndex:(NSInteger)index;
+ (NSArray<NSString *> *)codeIDArray;

// TypeOCR methods
+ (TypeOCR)ocrFromIndex:(NSInteger)index;
+ (NSArray<NSString *> *)ocrArray;

@end
