//
//  Store.m
//  AsReaderScan
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

#import "Store.h"
#import "ReaderModel.h"
#import "SettingModel.h"
#import "ParamModel.h"

@implementation Store

+ (instancetype)shared {
    static Store *sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [[self alloc] init];
    });
    return sharedInstance;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        //_tableData = [NSMutableArray array];
        _readerInfo = [[ReaderInfo alloc] init];
        _settingTable = [NSMutableArray arrayWithArray:DefaultItems()];
        _symbolTableFromArray = [NSMutableArray array];
        _securityTableFromArray = [NSMutableArray array];
        _codeIDType = CodeIDNone;

        [self loadSymbologyArray];
        [self loadSecurityArray];
    }
    return self;
}

- (void)loadSymbologyArray {
    NSArray *symbologyArray = [BarcodeZebraHelper getSymbologyArray];
    for (BarcodeZebraSsiParam *param in symbologyArray) {
        if ([param isKindOfClass:[BarcodeZebraSsiParam class]]) {
            SymbolItemFromArray *item = [[SymbolItemFromArray alloc] initWithTitle:param.name code:param.number];
            [self.symbolTableFromArray addObject:item];
        }
    }
}

- (void)loadSecurityArray {
    NSArray *securityArray = [BarcodeZebraHelper getSecurityArray];
    for (BarcodeZebraSsiParam *param in securityArray) {
        if ([param isKindOfClass:[BarcodeZebraSsiParam class]]) {
            SecurityItemFromArray *item = [[SecurityItemFromArray alloc] initWithTitle:param.name code:param.number];
            [self.securityTableFromArray addObject:item];
        }
    }
}

- (void)setAllSymbolEnable {
    for (NSInteger index = 0; index < self.symbolTableFromArray.count; index++) {
        self.symbolTableFromArray[index].isOn = YES;
    }
}

- (void)setAllSymbolDisable {
    for (NSInteger index = 0; index < self.symbolTableFromArray.count; index++) {
        self.symbolTableFromArray[index].isOn = false;
    }
}

- (void)setSymbolEnable:(NSString *)name isEnable:(BOOL)isEnable {
    NSInteger index = -1;
    
    for (NSInteger i = 0; i < self.symbolTableFromArray.count; i++) {
        if ([self.symbolTableFromArray[i].title isEqualToString:name]) {
            index = i;
            break; // Found the index, no need to continue
        }
    }
    
    if (index > -1) {
        self.symbolTableFromArray[index].isOn = isEnable;
    }
}

- (void)setSecurityLevel:(NSString *)name level:(NSInteger)level {
    NSInteger index = -1;

    for (NSInteger i = 0; i < self.securityTableFromArray.count; i++) {
        SecurityItemFromArray *item = self.securityTableFromArray[i];
        if ([item.title isEqualToString:name]) {
            index = i;
            break;
        }
    }

    if (index > -1) {
        SecurityItemFromArray *selectedItem = self.securityTableFromArray[index];
        selectedItem.level = level;
    }
}


//- (void)toggleSymbolSwitch:(SymbolItemFromArray *)item {
//    NSUInteger index = [self.symbolTableFromArray indexOfObject:item];
//    if (index != NSNotFound) {
//        item.isOn = !item.isOn;
//    }
//}
- (void)toggleSymbolSwitch:(SymbolItemFromArray *)item {
    NSUInteger index = [self.symbolTableFromArray indexOfObject:item];
    if (index != NSNotFound) {
        SymbolItemFromArray *symbolItem = self.symbolTableFromArray[index];
        symbolItem.isOn = !symbolItem.isOn;
    }
}

- (void)toggleSettingSwitch:(SettingItem *)item {
    NSUInteger index = [self.settingTable indexOfObject:item];
    if (index != NSNotFound) {
        SettingItem *settingItem = self.settingTable[index];
        settingItem.isOn = !settingItem.isOn; 
    }
}

//- (BOOL)updateTagDataWithTag:(NSString *)tag hexString:(NSString *)hexString header:(NSString *)header {
//    NSUInteger index = [self.tableData indexOfObjectPassingTest:^BOOL(TableItem *obj, NSUInteger idx, BOOL *stop) {
//        return [obj.data isEqualToString:tag];
//    }];
//    
//    if (index != NSNotFound) {
//        TableItem *item = self.tableData[index];
//        item.number += 1;
//
//        switch (self.codeIDType) {
//            case CodeIDNone:
//                item.header = @"";
//                break;
//            case CodeIDAim:
//            case CodeIDSymbol:
//                item.header = header;
//                break;
//        }
//        return NO;
//    } else {
//        TableItem *newItem = [[TableItem alloc] initWithData:tag number:1 hexData:hexString header:header];
//        [self.tableData addObject:newItem];
//        return YES;
//    }
//}

- (void)updateReaderInfo:(ReaderInfo *)info {
    self.readerInfo = info;
}
@end
