//
//  SettingViewController.m
//  AsReaderScan
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

#import "SettingViewController.h"
#import "SettingTableViewCell.h"

@implementation SettingViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self initial];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.presenter.delegate = self;
    self.presenter.navigationController = self.navigationController;
    [self.presenter getHIDMode];
    [self.presenter getPresentationMode];
    [self.presenter getSymbology];
}

// MARK: -- 초기화
- (void)initial {
    self.title = @"Reader Settings";
    self.presenter = [SettingPresenter sharedInstance];
	[self.presenter setDelegate:self];

    self.settingLists = self.presenter.model.settingTable;
    NSLog(@"Setting lists count: %lu", (unsigned long)self.settingLists.count);

    self.tableView.delegate = self;
    self.tableView.dataSource = self;
}

// MARK: TableView Delegate
#pragma mark - UITableViewDelegate & UITableViewDataSource

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.settingLists.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    SettingTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"settingCell" forIndexPath:indexPath];
    SettingItem *menuItem = self.settingLists[indexPath.row];

    cell.menuLabel.text = menuItem.title;

    switch (menuItem.controlType) {
        case switchType: {
            cell.switchOn.hidden = NO;
            cell.switchOn.on = menuItem.isOn;
            cell.switchOn.tag = menuItem.ids;
            //cell.switchOn.tag = (NSInteger)menuItem.ids;
            [cell.switchOn addTarget:self action:@selector(switchChanged:) forControlEvents:UIControlEventValueChanged];
            cell.btnSelect.hidden = YES;
            break;
        }
        case buttonType: {
            cell.btnSelect.hidden = NO;
            cell.btnSelect.tag = indexPath.row;
            [cell.btnSelect addTarget:self action:@selector(clickBtnSelect:) forControlEvents:UIControlEventTouchUpInside];
            cell.switchOn.hidden = YES;
            break;
        }
        default: {
            cell.switchOn.hidden = YES;
            cell.btnSelect.hidden = YES;
            break;
        }
    }
    return cell;
}

// MARK: -- Button Action
#pragma mark - Button Actions

- (void)updateTable {
    [self.tableView reloadData];
}

- (void)switchChanged:(UISwitch *)sender {
    NSLog(@"Switch changed: %@", sender.isOn ? @"ON" : @"OFF");
    [self.presenter changeSwitch:sender.isOn tagID:sender.tag];
}

- (void)clickBtnSelect:(UIButton *)sender {
    [self.presenter btnEvent:sender.tag viewController:self];
}

// MARK: -- 데이터가 변할때 UI 갱신을 위해 사용
#pragma mark - SettingPresenterDelegate

- (void)updatedReaderInfo {
    NSLog(@"Updated reader info called. Reloading table view.");
    [self.tableView reloadData];
}

@end

